package com.atguigu.tingshu.model.search;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import lombok.Data;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.CompletionField;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;
import org.springframework.data.elasticsearch.core.suggest.Completion;

@Data
@Document(indexName = "suggestinfo")
@JsonIgnoreProperties(ignoreUnknown = true)//目的：防止json字符串转成实体对象时因未识别字段报错
public class SuggestIndex {

    /**
     * 提词文档ID
     * 跟专辑ID一致：提词文档基于上架专辑同步
     */
    @Id
    private Long id;

    /**
     * 存储给用户返回中文提示词:百家讲坛，万历兴亡录
     */
    @Field(type = FieldType.Text, analyzer = "standard")
    private String title;

    /**
     * 用于建议补全检索字段： 录入:百家
     * 百家讲坛
     */
    @CompletionField(analyzer = "standard", searchAnalyzer = "standard", maxInputLength = 20)
    private Completion keyword;

    /**
     * 用于建议补全检索字段 录入:baijia
     * baijiajaingtan,wanlixingwanglu
     */
    @CompletionField(analyzer = "standard", searchAnalyzer = "standard", maxInputLength = 20)
    private Completion keywordPinyin;

    /**
     * 用于建议补全检索字段 录入:bj
     * bjjt,wlxwl
     */
    @CompletionField(analyzer = "standard", searchAnalyzer = "standard", maxInputLength = 20)
    private Completion keywordSequence;

}
