package com.atguigu.tingshu.album.api;

import com.alibaba.fastjson.JSONObject;
import com.atguigu.tingshu.album.service.BaseCategoryService;
import com.atguigu.tingshu.common.cache.GuiGuCache;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.model.album.BaseAttribute;
import com.atguigu.tingshu.model.album.BaseCategory1;
import com.atguigu.tingshu.model.album.BaseCategory3;
import com.atguigu.tingshu.model.album.BaseCategoryView;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


@Tag(name = "分类管理")
@RestController
@RequestMapping(value = "/api/album")
@SuppressWarnings({"all"})
public class BaseCategoryApiController {

    @Autowired
    private BaseCategoryService baseCategoryService;


    /**
     * 查询所有1,2,3级分类数据
     * TODO：后续将该业务数据放入Redis缓存中
     * @return
     */
    @Operation(summary = "查询所有1,2,3级分类数据")
    @GetMapping("/category/getBaseCategoryList")
    @GuiGuCache(prefix = "album:category:all")
    public Result<List<JSONObject>> getBaseCategoryList() {
        List<JSONObject> list = baseCategoryService.getBaseCategoryList();
        return Result.ok(list);
    }


    /**
     * 根据一级分类Id获取分类标签以及标签值
     * @param category1Id
     * @return
     */
    @Operation(summary = "根据一级分类Id获取分类标签以及标签值")
    @GetMapping("/category/findAttribute/{category1Id}")
    @GuiGuCache(prefix = "album:category:attribute:")
    public Result<List<BaseAttribute>> findAttributeByCategory1Id(@PathVariable("category1Id") Long category1Id) {
        List<BaseAttribute> list = baseCategoryService.findAttributeByCategory1Id(category1Id);
        return Result.ok(list);
    }


    /**
     * 根据三级分类ID查询分类信息
     * @param category3Id
     * @return
     */
    @Operation(summary = "根据三级分类ID查询分类信息")
    @GetMapping("/category/getCategoryView/{category3Id}")
    @GuiGuCache(prefix = "album:category:category3:")
    public Result<BaseCategoryView> getCategoryView(@PathVariable("category3Id") Long category3Id) {
        BaseCategoryView baseCategoryView = baseCategoryService.getCategoryView(category3Id);
        return Result.ok(baseCategoryView);
    }

    /**
     * 根据1级分类ID查询置顶7个三级分类列表
     * @param category1Id
     * @return
     */
    @Operation(summary = "根据1级分类ID查询置顶7个三级分类列表")
    @GetMapping("/category/findTopBaseCategory3/{category1Id}")
    @GuiGuCache(prefix = "album:category:top3:")
    public Result<List<BaseCategory3>> findTopBaseCategory3(@PathVariable("category1Id") Long category1Id) {
        List<BaseCategory3> list = baseCategoryService.findTopBaseCategory3(category1Id);
        return Result.ok(list);
    }


    /**
     * 根据1级分类ID查询到1级分类对象（包含所属二级分类，所属三级分类列表）
     * @param category1Id
     * @return
     */
    @Operation(summary = "根据1级分类ID查询到1级分类对象（包含所属二级分类，所属三级分类列表）")
    @GetMapping("/category/getBaseCategoryList/{category1Id}")
    @GuiGuCache(prefix = "album:category:")
    public Result<JSONObject> getBaseCategoryListByCategory1Id(@PathVariable("category1Id") Long category1Id) {
        JSONObject jsonObject = baseCategoryService.getBaseCategoryListByCategory1Id(category1Id);
        return Result.ok(jsonObject);
    }

    /**
     * 查询所有一级分类列表
     *
     * @return
     */
    @Operation(summary = "查询所有一级分类列表")
    @GetMapping("/category/findAllCategory1")
    @GuiGuCache(prefix = "album:category:category1")
    public Result<List<BaseCategory1>> getAllCategory1() {
        LambdaQueryWrapper<BaseCategory1> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.select(BaseCategory1::getId);
        List<BaseCategory1> list = baseCategoryService.list(queryWrapper);
        return Result.ok(list);
    }
}

