package com.atguigu.tingshu.album.service;

import com.atguigu.tingshu.model.album.TrackInfo;
import com.atguigu.tingshu.query.album.TrackInfoQuery;
import com.atguigu.tingshu.vo.album.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;
import java.util.Map;

public interface TrackInfoService extends IService<TrackInfo> {

    /**
     * 保存声音，发起内容审核任务
     * @param trackInfoVo
     * @param userId
     */
    void saveTrackInfo(TrackInfoVo trackInfoVo, Long userId);

    /**
     * 保存声音统计信息
     * @param trackId 声音ID
     * @param statType 统计类型
     * @param statNum 统计数值
     */
    void saveTrackStat(Long trackId, String statType, int statNum);

    /**
     * 获取当前用户声音分页列表
     *
     * @param pageInfo
     * @param trackInfoQuery
     * @return
     */
    Page<TrackListVo> findUserTrackPage(Page<TrackListVo> pageInfo, TrackInfoQuery trackInfoQuery);

    void updateTrackInfo(Long id, TrackInfoVo trackInfoVo);

    /**
     * 删除声音
     * @param id 声音ID
     * @return
     */
    void removeTrackInfo(Long id);

    /**
     * 分页查询指定专辑下声音列表，综合根据登录状态、用户身份、购买情况显示付费标识
     * @param pageInfo 分页对象
     * @param albumId 专辑ID
     * @param userId 用户ID
     * @return
     */
    Page<AlbumTrackListVo> findAlbumTrackPage(Page<AlbumTrackListVo> pageInfo, Long albumId, Long userId);

    /**
     * 查询声音统计信息
     * @param trackId
     * @return
     */
    TrackStatVo getTrackStatVo(Long trackId);

    /**
     * 更新数据库中声音表以及专辑表统计数值
     * @param mqVo
     */
    void updateTrackStat(TrackStatMqVo mqVo);

    /**
     * 基于用户未购买声音数量动态构建分集购买列表
     * @param trackId 选择购买（起始位置）声音ID
     * @return {data:[{name:"本集",price:0.2,trackCount:1},{name:"后10集",price:2,trackCount:10},{...},{name:"全集",price:4.4,trackCount:22}]}
     */
    List<Map<String, Object>> findUserTrackPaidList(Long userId, Long trackId);

    /**
     * 查询指定用户未购买声音列表
     * @param userId 用户ID
     * @param trackId 声音ID 作为起始声音
     * @param trackCount 声音数量
     * @return
     */
    List<TrackInfo> findPaidTrackInfoList(Long userId, Long trackId, Integer trackCount);
}
