package com.atguigu.tingshu.user.api;

import com.atguigu.tingshu.common.cache.GuiGuCache;
import com.atguigu.tingshu.common.login.GuiGuLogin;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.user.service.UserInfoService;
import com.atguigu.tingshu.vo.user.UserInfoVo;
import com.atguigu.tingshu.vo.user.UserPaidRecordVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Tag(name = "用户管理接口")
@RestController
@RequestMapping("api/user")
@SuppressWarnings({"all"})
public class UserInfoApiController {

    @Autowired
    private UserInfoService userInfoService;

    @Operation(summary = "查询用户基本信息")
    @GetMapping("/userInfo/getUserInfoVo/{userId}")
    @GuiGuCache(prefix = "user:userinfo:")
    public Result<UserInfoVo> getUserInfoVo(@PathVariable("userId") Long userId) {
        UserInfoVo userInfoVo = userInfoService.getUserInfo(userId);
        return Result.ok(userInfoVo);
    }


    /**
     * 根据用户ID+专辑ID+声音ID列表查询得到声音购买情况
     *
     * @param userId
     * @param albumId
     * @param needCheckBuyStateTrackIds
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "根据专辑ID+声音ID列表查询得到声音购买情况")
    @PostMapping("/userInfo/userIsPaidTrack/{userId}/{albumId}")
    public Result<Map<Long, Integer>> userIsPaidTrack(
            @PathVariable("userId") Long userId,
            @PathVariable("albumId") Long albumId,
            @RequestBody List<Long> needCheckBuyStateTrackIds
    ) {
        Map<Long, Integer> map = userInfoService.userIsPaidTrack(userId, albumId, needCheckBuyStateTrackIds);
        return Result.ok(map);
    }


    /**
     * 验证当前用户是否购买过指定专辑
     *
     * @param albumId
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "验证当前用户是否购买过指定专辑")
    @GetMapping("/userInfo/isPaidAlbum/{albumId}")
    public Result<Boolean> isPaidAlbum(@PathVariable("albumId") Long albumId) {
        //1.获取当前用户ID
        Long userId = AuthContextHolder.getUserId();
        //2.调用业务层方法返回指定用户指定专辑购买结果
        boolean isPaidAlbum = userInfoService.isPaidAlbum(userId, albumId);
        return Result.ok(isPaidAlbum);
    }

    /**
     * 查询用户指定专辑已购声音ID列表
     * @param albumId
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "查询用户指定专辑已购声音ID列表")
    @GetMapping("/userInfo/findUserPaidTrackList/{albumId}")
    public Result<List<Long>> findUserPaidTrackList(@PathVariable("albumId") Long albumId) {
        Long userId = AuthContextHolder.getUserId();
        List<Long> list = userInfoService.findUserPaidTrackList(userId, albumId);
        return Result.ok(list);
    }


    /**
     * 用户付款成功后，虚拟物品发货
     * @param userPaidRecordVo
     * @return
     */
    @Operation(summary = "用户付款成功后，虚拟物品发货")
    @PostMapping("/userInfo/savePaidRecord")
    public Result savePaidRecord(@RequestBody UserPaidRecordVo userPaidRecordVo){
        userInfoService.savePaidRecord(userPaidRecordVo);
        return Result.ok();
    }
}

